@@ -53,6 +53,50 @@ namespace {
   } while (false)
 }  // namespace
 
+//------------------------------------------------------------
+// DriverQuirks
+
+AdbcStatusCode DoIngestSampleTable(struct AdbcConnection* connection,
+                                   const std::string& name, struct AdbcError* error) {
+  Handle<struct ArrowSchema> schema;
+  Handle<struct ArrowArray> array;
+  struct ArrowError na_error;
+  CHECK_OK(MakeSchema(&schema.value, {{"int64s", NANOARROW_TYPE_INT64},
+                                      {"strings", NANOARROW_TYPE_STRING}}));
+  CHECK_OK((MakeBatch<int64_t, std::string>(&schema.value, &array.value, &na_error,
+                                            {42, -42, std::nullopt},
+                                            {"foo", std::nullopt, ""})));
+
+  Handle<struct AdbcStatement> statement;
+  CHECK_OK(AdbcStatementNew(connection, &statement.value, error));
+  CHECK_OK(AdbcStatementSetOption(&statement.value, ADBC_INGEST_OPTION_TARGET_TABLE,
+                                  name.c_str(), error));
+  CHECK_OK(AdbcStatementBind(&statement.value, &array.value, &schema.value, error));
+  CHECK_OK(AdbcStatementExecuteQuery(&statement.value, nullptr, nullptr, error));
+  CHECK_OK(AdbcStatementRelease(&statement.value, error));
+  return ADBC_STATUS_OK;
+}
+
+void IngestSampleTable(struct AdbcConnection* connection, struct AdbcError* error) {
+  ASSERT_THAT(DoIngestSampleTable(connection, "bulk_ingest", error), IsOkStatus(error));
+}
+
+AdbcStatusCode DriverQuirks::EnsureSampleTable(struct AdbcConnection* connection,
+                                               const std::string& name,
+                                               struct AdbcError* error) const {
+  CHECK_OK(DropTable(connection, name, error));
+  return CreateSampleTable(connection, name, error);
+}
+
+AdbcStatusCode DriverQuirks::CreateSampleTable(struct AdbcConnection* connection,
+                                               const std::string& name,
+                                               struct AdbcError* error) const {
+  if (!supports_bulk_ingest()) {
+    return ADBC_STATUS_NOT_IMPLEMENTED;
+  }
+  return DoIngestSampleTable(connection, name, error);
+}
+
 //------------------------------------------------------------
 // Tests of AdbcDatabase
 
@@ -173,6 +217,9 @@ void ConnectionTest::TestAutocommitDefault() {
 void ConnectionTest::TestAutocommitToggle() {
   ASSERT_THAT(AdbcConnectionNew(&connection, &error), IsOkStatus(&error));
   ASSERT_THAT(AdbcConnectionInit(&connection, &database, &error), IsOkStatus(&error));
+  if (!quirks()->supports_transactions()) {
+    GTEST_SKIP();
+  }
 
   // It is OK to enable autocommit when it is already enabled
   ASSERT_THAT(AdbcConnectionSetOption(&connection, ADBC_CONNECTION_OPTION_AUTOCOMMIT,
@@ -190,34 +237,14 @@ void ConnectionTest::TestAutocommitToggle() {
 //------------------------------------------------------------
 // Tests of metadata
 
-void IngestSampleTable(struct AdbcConnection* connection, struct AdbcError* error) {
-  Handle<struct ArrowSchema> schema;
-  Handle<struct ArrowArray> array;
-  struct ArrowError na_error;
-  ASSERT_THAT(MakeSchema(&schema.value, {{"int64s", NANOARROW_TYPE_INT64},
-                                         {"strings", NANOARROW_TYPE_STRING}}),
-              IsOkErrno());
-  ASSERT_THAT((MakeBatch<int64_t, std::string>(&schema.value, &array.value, &na_error,
-                                               {42, -42, std::nullopt},
-                                               {"foo", std::nullopt, ""})),
-              IsOkErrno());
-
-  Handle<struct AdbcStatement> statement;
-  ASSERT_THAT(AdbcStatementNew(connection, &statement.value, error), IsOkStatus(error));
-  ASSERT_THAT(AdbcStatementSetOption(&statement.value, ADBC_INGEST_OPTION_TARGET_TABLE,
-                                     "bulk_ingest", error),
-              IsOkStatus(error));
-  ASSERT_THAT(AdbcStatementBind(&statement.value, &array.value, &schema.value, error),
-              IsOkStatus(error));
-  ASSERT_THAT(AdbcStatementExecuteQuery(&statement.value, nullptr, nullptr, error),
-              IsOkStatus(error));
-  ASSERT_THAT(AdbcStatementRelease(&statement.value, error), IsOkStatus(error));
-}
-
 void ConnectionTest::TestMetadataGetInfo() {
   ASSERT_THAT(AdbcConnectionNew(&connection, &error), IsOkStatus(&error));
   ASSERT_THAT(AdbcConnectionInit(&connection, &database, &error), IsOkStatus(&error));
 
+  if (!quirks()->supports_get_sql_info()) {
+    GTEST_SKIP();
+  }
+
   StreamReader reader;
   std::vector<uint32_t> info = {
       ADBC_INFO_DRIVER_NAME,
@@ -275,6 +302,9 @@ void ConnectionTest::TestMetadataGetInfo() {
 }
 
 void ConnectionTest::TestMetadataGetTableSchema() {
+  if (!quirks()->supports_bulk_ingest()) {
+    GTEST_SKIP();
+  }
   ASSERT_THAT(AdbcConnectionNew(&connection, &error), IsOkStatus(&error));
   ASSERT_THAT(AdbcConnectionInit(&connection, &database, &error), IsOkStatus(&error));
   ASSERT_THAT(quirks()->DropTable(&connection, "bulk_ingest", &error),
@@ -376,6 +406,10 @@ void ConnectionTest::TestMetadataGetObjectsCatalogs() {
   ASSERT_THAT(AdbcConnectionNew(&connection, &error), IsOkStatus(&error));
   ASSERT_THAT(AdbcConnectionInit(&connection, &database, &error), IsOkStatus(&error));
 
+  if (!quirks()->supports_get_objects()) {
+    GTEST_SKIP();
+  }
+
   {
     StreamReader reader;
     ASSERT_THAT(AdbcConnectionGetObjects(&connection, ADBC_OBJECT_DEPTH_CATALOGS, nullptr,
@@ -420,6 +454,10 @@ void ConnectionTest::TestMetadataGetObjectsDbSchemas() {
   ASSERT_THAT(AdbcConnectionNew(&connection, &error), IsOkStatus(&error));
   ASSERT_THAT(AdbcConnectionInit(&connection, &database, &error), IsOkStatus(&error));
 
+  if (!quirks()->supports_get_objects()) {
+    GTEST_SKIP();
+  }
+
   {
     // Expect at least one catalog, at least one schema, and tables should be null
     StreamReader reader;
@@ -491,9 +529,12 @@ void ConnectionTest::TestMetadataGetObjectsDbSchemas() {
 void ConnectionTest::TestMetadataGetObjectsTables() {
   ASSERT_THAT(AdbcConnectionNew(&connection, &error), IsOkStatus(&error));
   ASSERT_THAT(AdbcConnectionInit(&connection, &database, &error), IsOkStatus(&error));
-  ASSERT_THAT(quirks()->DropTable(&connection, "bulk_ingest", &error),
+  if (!quirks()->supports_get_objects()) {
+    GTEST_SKIP();
+  }
+
+  ASSERT_THAT(quirks()->EnsureSampleTable(&connection, "bulk_ingest", &error),
               IsOkStatus(&error));
-  ASSERT_NO_FATAL_FAILURE(IngestSampleTable(&connection, &error));
 
   std::vector<std::pair<const char*, bool>> test_cases = {
       {nullptr, true}, {"bulk_%", true}, {"asdf%", false}};
@@ -567,9 +608,12 @@ void ConnectionTest::TestMetadataGetObjectsTables() {
 void ConnectionTest::TestMetadataGetObjectsTablesTypes() {
   ASSERT_THAT(AdbcConnectionNew(&connection, &error), IsOkStatus(&error));
   ASSERT_THAT(AdbcConnectionInit(&connection, &database, &error), IsOkStatus(&error));
-  ASSERT_THAT(quirks()->DropTable(&connection, "bulk_ingest", &error),
+  if (!quirks()->supports_get_objects()) {
+    GTEST_SKIP();
+  }
+
+  ASSERT_THAT(quirks()->EnsureSampleTable(&connection, "bulk_ingest", &error),
               IsOkStatus(&error));
-  ASSERT_NO_FATAL_FAILURE(IngestSampleTable(&connection, &error));
 
   std::vector<const char*> table_types(2);
   table_types[0] = "this_table_type_does_not_exist";
@@ -634,14 +678,16 @@ void ConnectionTest::TestMetadataGetObjectsTablesTypes() {
 }
 
 void ConnectionTest::TestMetadataGetObjectsColumns() {
+  if (!quirks()->supports_get_objects()) {
+    GTEST_SKIP();
+  }
   // TODO: test could be more robust if we ingested a few tables
   ASSERT_EQ(ADBC_OBJECT_DEPTH_COLUMNS, ADBC_OBJECT_DEPTH_ALL);
 
   ASSERT_THAT(AdbcConnectionNew(&connection, &error), IsOkStatus(&error));
   ASSERT_THAT(AdbcConnectionInit(&connection, &database, &error), IsOkStatus(&error));
-  ASSERT_THAT(quirks()->DropTable(&connection, "bulk_ingest", &error),
+  ASSERT_THAT(quirks()->EnsureSampleTable(&connection, "bulk_ingest", &error),
               IsOkStatus(&error));
-  ASSERT_NO_FATAL_FAILURE(IngestSampleTable(&connection, &error));
 
   struct TestCase {
     std::optional<std::string> filter;
@@ -800,6 +846,10 @@ void StatementTest::TestRelease() {
 }
 
 void StatementTest::TestSqlIngestInts() {
+  if (!quirks()->supports_bulk_ingest()) {
+    GTEST_SKIP();
+  }
+
   ASSERT_THAT(quirks()->DropTable(&connection, "bulk_ingest", &error),
               IsOkStatus(&error));
 
@@ -851,6 +901,10 @@ void StatementTest::TestSqlIngestInts() {
 }
 
 void StatementTest::TestSqlIngestAppend() {
+  if (!quirks()->supports_bulk_ingest()) {
+    GTEST_SKIP();
+  }
+
   // Ingest
   ASSERT_THAT(quirks()->DropTable(&connection, "bulk_ingest", &error),
               IsOkStatus(&error));
@@ -926,6 +980,10 @@ void StatementTest::TestSqlIngestAppend() {
 }
 
 void StatementTest::TestSqlIngestErrors() {
+  if (!quirks()->supports_bulk_ingest()) {
+    GTEST_SKIP();
+  }
+
   // Ingest without bind
   ASSERT_THAT(AdbcStatementNew(&connection, &statement, &error), IsOkStatus(&error));
   ASSERT_THAT(AdbcStatementSetOption(&statement, ADBC_INGEST_OPTION_TARGET_TABLE,
@@ -1002,6 +1060,10 @@ void StatementTest::TestSqlIngestErrors() {
 }
 
 void StatementTest::TestSqlIngestMultipleConnections() {
+  if (!quirks()->supports_bulk_ingest()) {
+    GTEST_SKIP();
+  }
+
   ASSERT_THAT(quirks()->DropTable(&connection, "bulk_ingest", &error),
               IsOkStatus(&error));
 
@@ -1077,7 +1139,7 @@ void StatementTest::TestSqlPartitionedInts() {
     ASSERT_THAT(AdbcStatementExecutePartitions(&statement, &schema.value,
                                                &partitions.value, &rows_affected, &error),
                 IsStatus(ADBC_STATUS_NOT_IMPLEMENTED, &error));
-    return;
+    GTEST_SKIP();
   }
 
   ASSERT_THAT(AdbcStatementExecutePartitions(&statement, &schema.value, &partitions.value,
@@ -1086,8 +1148,10 @@ void StatementTest::TestSqlPartitionedInts() {
   // Assume only 1 partition
   ASSERT_EQ(1, partitions->num_partitions);
   ASSERT_THAT(rows_affected, ::testing::AnyOf(::testing::Eq(1), ::testing::Eq(-1)));
-  ASSERT_NE(nullptr, schema->release);
-  ASSERT_EQ(1, schema->n_children);
+  // it's allowed for Executepartitions to return a nil schema if one is not available
+  if (schema->release != nullptr) {
+    ASSERT_EQ(1, schema->n_children);
+  }
 
   Handle<struct AdbcConnection> connection2;
   StreamReader reader;
@@ -1125,19 +1189,28 @@ void StatementTest::TestSqlPartitionedInts() {
 }
 
 void StatementTest::TestSqlPrepareGetParameterSchema() {
+  if (!quirks()->supports_dynamic_parameter_binding()) {
+    GTEST_SKIP();
+  }
+
   ASSERT_THAT(AdbcStatementNew(&connection, &statement, &error), IsOkStatus(&error));
   std::string query = "SELECT ";
   query += quirks()->BindParameter(0);
   query += ", ";
   query += quirks()->BindParameter(1);
+
   ASSERT_THAT(AdbcStatementSetSqlQuery(&statement, query.c_str(), &error),
               IsOkStatus(&error));
   ASSERT_THAT(AdbcStatementPrepare(&statement, &error), IsOkStatus(&error));
 
   Handle<struct ArrowSchema> schema;
+  // if schema cannot be determined we should get NOT IMPLEMENTED returned
   ASSERT_THAT(AdbcStatementGetParameterSchema(&statement, &schema.value, &error),
-              IsOkStatus(&error));
-  ASSERT_EQ(2, schema->n_children);
+              ::testing::AnyOf(IsOkStatus(&error),
+                               IsStatus(ADBC_STATUS_NOT_IMPLEMENTED, &error)));
+  if (schema->release != nullptr) {
+    ASSERT_EQ(2, schema->n_children);
+  }
   // Can't assume anything about names or types here
 }
 
@@ -1178,6 +1251,10 @@ void StatementTest::TestSqlPrepareSelectNoParams() {
 }
 
 void StatementTest::TestSqlPrepareSelectParams() {
+  if (!quirks()->supports_dynamic_parameter_binding()) {
+    GTEST_SKIP();
+  }
+
   ASSERT_THAT(AdbcStatementNew(&connection, &statement, &error), IsOkStatus(&error));
   std::string query = "SELECT ";
   query += quirks()->BindParameter(0);
@@ -1210,31 +1287,49 @@ void StatementTest::TestSqlPrepareSelectParams() {
   ASSERT_NO_FATAL_FAILURE(reader.GetSchema());
   ASSERT_EQ(2, reader.schema->n_children);
 
-  ASSERT_NO_FATAL_FAILURE(reader.Next());
-  ASSERT_NE(nullptr, reader.array->release);
-  ASSERT_EQ(3, reader.array->length);
-  ASSERT_EQ(2, reader.array->n_children);
+  const std::vector<std::optional<int32_t>> expected_int32{42, -42, std::nullopt};
+  const std::vector<std::optional<int64_t>> expected_int64{42, -42, std::nullopt};
+  const std::vector<std::optional<std::string>> expected_string{"", std::nullopt, "bar"};
 
-  switch (reader.fields[0].data_type) {
-    case NANOARROW_TYPE_INT32:
-      ASSERT_NO_FATAL_FAILURE(
-          CompareArray<int32_t>(reader.array_view->children[0], {42, -42, std::nullopt}));
-      break;
-    case NANOARROW_TYPE_INT64:
-      ASSERT_NO_FATAL_FAILURE(
-          CompareArray<int64_t>(reader.array_view->children[0], {42, -42, std::nullopt}));
-      break;
-    default:
-      FAIL() << "Unexpected data type: " << reader.fields[0].data_type;
+  int64_t nrows = 0;
+  while (nrows < 3) {
+    ASSERT_NO_FATAL_FAILURE(reader.Next());
+    ASSERT_NE(nullptr, reader.array->release);
+    ASSERT_EQ(2, reader.array->n_children);
+
+    auto start = nrows;
+    auto end = nrows + reader.array->length;
+
+    switch (reader.fields[0].data_type) {
+      case NANOARROW_TYPE_INT32:
+        ASSERT_NO_FATAL_FAILURE(CompareArray<int32_t>(
+            reader.array_view->children[0],
+            {expected_int32.begin() + start, expected_int32.begin() + end}));
+        break;
+      case NANOARROW_TYPE_INT64:
+        ASSERT_NO_FATAL_FAILURE(CompareArray<int64_t>(
+            reader.array_view->children[0],
+            {expected_int64.begin() + start, expected_int64.begin() + end}));
+        break;
+      default:
+        FAIL() << "Unexpected data type: " << reader.fields[0].data_type;
+    }
+    ASSERT_NO_FATAL_FAILURE(CompareArray<std::string>(
+        reader.array_view->children[1],
+        {expected_string.begin() + start, expected_string.begin() + end}));
+    nrows += reader.array->length;
   }
-  ASSERT_NO_FATAL_FAILURE(CompareArray<std::string>(reader.array_view->children[1],
-                                                    {"", std::nullopt, "bar"}));
+  ASSERT_EQ(3, nrows);
 
   ASSERT_NO_FATAL_FAILURE(reader.Next());
   ASSERT_EQ(nullptr, reader.array->release);
 }
 
 void StatementTest::TestSqlPrepareUpdate() {
+  if (!quirks()->supports_bulk_ingest()) {
+    GTEST_SKIP();
+  }
+
   ASSERT_THAT(AdbcStatementNew(&connection, &statement, &error), IsOkStatus(&error));
   ASSERT_THAT(quirks()->DropTable(&connection, "bulk_ingest", &error),
               IsOkStatus(&error));
@@ -1309,6 +1404,10 @@ void StatementTest::TestSqlPrepareUpdateNoParams() {
 }
 
 void StatementTest::TestSqlPrepareUpdateStream() {
+  if (!quirks()->supports_bulk_ingest()) {
+    GTEST_SKIP();
+  }
+
   ASSERT_THAT(AdbcStatementNew(&connection, &statement, &error), IsOkStatus(&error));
   ASSERT_THAT(quirks()->DropTable(&connection, "bulk_ingest", &error),
               IsOkStatus(&error));
@@ -1406,6 +1505,10 @@ void StatementTest::TestSqlPrepareErrorNoQuery() {
 // TODO: need test of overlapping reads - make sure behavior is as described
 
 void StatementTest::TestSqlPrepareErrorParamCountMismatch() {
+  if (!quirks()->supports_dynamic_parameter_binding()) {
+    GTEST_SKIP();
+  }
+
   Handle<struct ArrowSchema> schema;
   Handle<struct ArrowArray> array;
   struct ArrowError na_error;
@@ -1478,7 +1581,7 @@ void StatementTest::TestSqlQueryInts() {
 
 void StatementTest::TestSqlQueryFloats() {
   ASSERT_THAT(AdbcStatementNew(&connection, &statement, &error), IsOkStatus(&error));
-  ASSERT_THAT(AdbcStatementSetSqlQuery(&statement, "SELECT CAST(1.0 AS REAL)", &error),
+  ASSERT_THAT(AdbcStatementSetSqlQuery(&statement, "SELECT CAST(1.0 AS FLOAT)", &error),
               IsOkStatus(&error));
 
   {
@@ -1571,6 +1674,10 @@ void StatementTest::TestSqlQueryErrors() {
 }
 
 void StatementTest::TestTransactions() {
+  if (!quirks()->supports_transactions()) {
+    GTEST_SKIP();
+  }
+
   ASSERT_THAT(quirks()->DropTable(&connection, "bulk_ingest", &error),
               IsOkStatus(&error));
 
