@@ -36,6 +36,21 @@ main() {
 
     go build -v ./...
 
+    if [[ "${CGO_ENABLED}" = 1 ]]; then
+        pushd ./pkg
+
+        make all
+
+        popd
+
+        mkdir -p "${install_dir}/lib"
+        if [[ $(go env GOOS) == "linux" ]]; then
+            cp ./pkg/libadbc_driver_flightsql.so "${install_dir}/lib"
+        else
+            cp ./pkg/libadbc_driver_flightsql.dylib "${install_dir}/lib"
+        fi
+    fi
+
     popd
 }
 
