@@ -89,7 +89,7 @@ show_info() {
 ARROW_DIST_URL='https://dist.apache.org/repos/dist/dev/arrow'
 
 download_dist_file() {
-  if [[ -n "${VERIFICATION_MOCK_DIST_DIR}" && "$1" != "KEYS" ]]; then
+  if [[ -n "${VERIFICATION_MOCK_DIST_DIR}" ]]; then
     cp "${VERIFICATION_MOCK_DIST_DIR}/$1" .
   else
     curl \
@@ -116,10 +116,8 @@ import_gpg_keys() {
 }
 
 if type shasum >/dev/null 2>&1; then
-  sha256_verify="shasum -a 256 -c"
   sha512_verify="shasum -a 512 -c"
 else
-  sha256_verify="sha256sum -c"
   sha512_verify="sha512sum -c"
 fi
 
@@ -129,10 +127,8 @@ fetch_archive() {
   local dist_name=$1
   download_rc_file ${dist_name}.tar.gz
   download_rc_file ${dist_name}.tar.gz.asc
-  download_rc_file ${dist_name}.tar.gz.sha256
   download_rc_file ${dist_name}.tar.gz.sha512
   gpg --verify ${dist_name}.tar.gz.asc ${dist_name}.tar.gz
-  ${sha256_verify} ${dist_name}.tar.gz.sha256
   ${sha512_verify} ${dist_name}.tar.gz.sha512
 }
 
@@ -148,12 +144,7 @@ verify_dir_artifact_signatures() {
     # basename of the artifact
     pushd $(dirname $artifact) >/dev/null
     base_artifact=$(basename $artifact)
-    if [ -f $base_artifact.sha256 ]; then
-      ${sha256_verify} $base_artifact.sha256 || exit 1
-    fi
-    if [ -f $base_artifact.sha512 ]; then
-      ${sha512_verify} $base_artifact.sha512 || exit 1
-    fi
+    ${sha512_verify} $base_artifact.sha512 || exit 1
     popd >/dev/null
   done
 }
@@ -227,6 +218,8 @@ test_yum() {
 setup_tempdir() {
   cleanup() {
     if [ "${TEST_SUCCESS}" = "yes" ]; then
+      # Go modules are installed with 0444.
+      chmod -R u+w "${ARROW_TMPDIR}"
       rm -fr "${ARROW_TMPDIR}"
     else
       echo "Failed to verify release candidate. See ${ARROW_TMPDIR} for details."
@@ -414,9 +407,11 @@ test_cpp() {
   show_header "Build, install and test C++ libraries"
 
   # Build and test C++
+  maybe_setup_go
   maybe_setup_conda \
     --file ci/conda_env_cpp.txt \
-    compilers || exit 1
+    compilers \
+    go=1.18 || exit 1
 
   if [ "${USE_CONDA}" -gt 0 ]; then
     export CMAKE_PREFIX_PATH="${CONDA_BACKUP_CMAKE_PREFIX_PATH}:${CMAKE_PREFIX_PATH}"
@@ -433,9 +428,12 @@ test_cpp() {
   export ADBC_USE_ASAN=OFF
   export ADBC_USE_UBSAN=OFF
   "${ADBC_SOURCE_DIR}/ci/scripts/cpp_build.sh" "${ADBC_SOURCE_DIR}" "${ARROW_TMPDIR}/cpp-build" "${install_prefix}"
+  # FlightSQL driver requires running database for testing
+  export BUILD_DRIVER_FLIGHTSQL=0
   # PostgreSQL driver requires running database for testing
   export BUILD_DRIVER_POSTGRESQL=0
   "${ADBC_SOURCE_DIR}/ci/scripts/cpp_test.sh" "${ADBC_SOURCE_DIR}" "${ARROW_TMPDIR}/cpp-build" "${install_prefix}"
+  export BUILD_DRIVER_FLIGHTSQL=1
   export BUILD_DRIVER_POSTGRESQL=1
 }
 
@@ -482,6 +480,9 @@ test_glib() {
 
   # Install bundler if doesn't exist
   if ! bundle --version; then
+    # This is for old Ruby. Recent Ruby bundles Bundler.
+    export GEM_HOME="${ARROW_TMPDIR}/glib-build/gems"
+    export PATH="${GEM_HOME}/bin:${PATH}"
     gem install --no-document bundler
   fi
 
@@ -532,8 +533,8 @@ test_go() {
 
 
   export CGO_ENABLED=1
-  "${ADBC_SOURCE_DIR}/ci/scripts/go_build.sh" "${ADBC_SOURCE_DIR}" "${ARROW_TMPDIR}/glib-build" "${install_prefix}"
-  "${ADBC_SOURCE_DIR}/ci/scripts/go_test.sh" "${ADBC_SOURCE_DIR}" "${ARROW_TMPDIR}/glib-build" "${install_prefix}"
+  "${ADBC_SOURCE_DIR}/ci/scripts/go_build.sh" "${ADBC_SOURCE_DIR}" "${ARROW_TMPDIR}/go-build" "${install_prefix}"
+  "${ADBC_SOURCE_DIR}/ci/scripts/go_test.sh" "${ADBC_SOURCE_DIR}" "${ARROW_TMPDIR}/go-build" "${install_prefix}"
 }
 
 ensure_source_directory() {
