@@ -40,11 +40,12 @@ import (
 	"context"
 	"fmt"
 
-	"github.com/apache/arrow/go/v10/arrow"
-	"github.com/apache/arrow/go/v10/arrow/array"
+	"github.com/apache/arrow/go/v12/arrow"
+	"github.com/apache/arrow/go/v12/arrow/array"
 )
 
 //go:generate go run golang.org/x/tools/cmd/stringer -type Status -linecomment
+//go:generate go run golang.org/x/tools/cmd/stringer -type InfoCode -linecomment
 
 // Error is the detailed error for an operation
 type Error struct {
@@ -152,6 +153,9 @@ const (
 	OptionKeyReadOnly           = "adbc.connection.readonly"
 	OptionValueIngestModeCreate = "adbc.ingest.mode.create"
 	OptionValueIngestModeAppend = "adbc.ingest.mode.append"
+	OptionKeyURI                = "uri"
+	OptionKeyUsername           = "username"
+	OptionKeyPassword           = "password"
 )
 
 type OptionIsolationLevel string
@@ -183,6 +187,10 @@ const (
 // A driver can also optionally implement io.Closer if there is a need
 // or desire for it.
 type Driver interface {
+	NewDatabase(opts map[string]string) (Database, error)
+}
+
+type Database interface {
 	SetOptions(map[string]string) error
 	Open(ctx context.Context) (Connection, error)
 }
@@ -192,18 +200,18 @@ type InfoCode uint32
 const (
 	// The database vendor/product name (e.g. the server name)
 	// (type: utf8)
-	InfoVendorName InfoCode = 0
+	InfoVendorName InfoCode = 0 // VendorName
 	// The database vendor/product version (type: utf8)
-	InfoVendorVersion InfoCode = 1
+	InfoVendorVersion InfoCode = 1 // VendorVersion
 	// The database vendor/product Arrow library version (type: utf8)
-	InfoVendorArrowVersion InfoCode = 2
+	InfoVendorArrowVersion InfoCode = 2 // VendorArrowVersion
 
 	// The driver name (type: utf8)
-	InfoDriverName InfoCode = 100
+	InfoDriverName InfoCode = 100 // DriverName
 	// The driver version (type: utf8)
-	InfoDriverVersion InfoCode = 101
+	InfoDriverVersion InfoCode = 101 // DriverVersion
 	// The driver Arrow library version (type: utf8)
-	InfoDriverArrowVersion InfoCode = 102
+	InfoDriverArrowVersion InfoCode = 102 // DriverArrowVersion
 )
 
 type ObjectDepth int
