@@ -40,14 +40,15 @@ import (
 	"reflect"
 	"strconv"
 	"strings"
+	"time"
 	"unsafe"
 
 	"github.com/apache/arrow-adbc/go/adbc"
-	"github.com/apache/arrow/go/v10/arrow"
-	"github.com/apache/arrow/go/v10/arrow/array"
-	"github.com/apache/arrow/go/v10/arrow/decimal128"
-	"github.com/apache/arrow/go/v10/arrow/decimal256"
-	"github.com/apache/arrow/go/v10/arrow/memory"
+	"github.com/apache/arrow/go/v12/arrow"
+	"github.com/apache/arrow/go/v12/arrow/array"
+	"github.com/apache/arrow/go/v12/arrow/decimal128"
+	"github.com/apache/arrow/go/v12/arrow/decimal256"
+	"github.com/apache/arrow/go/v12/arrow/memory"
 )
 
 func getIsolationlevel(lvl sql.IsolationLevel) adbc.OptionIsolationLevel {
@@ -73,7 +74,7 @@ func getIsolationlevel(lvl sql.IsolationLevel) adbc.OptionIsolationLevel {
 func parseConnectStr(str string) (ret map[string]string, err error) {
 	ret = make(map[string]string)
 	for _, kv := range strings.Split(str, ";") {
-		parsed := strings.Split(kv, "=")
+		parsed := strings.SplitN(kv, "=", 2)
 		if len(parsed) != 2 {
 			return nil, &adbc.Error{
 				Msg:  "invalid format for connection string",
@@ -87,7 +88,8 @@ func parseConnectStr(str string) (ret map[string]string, err error) {
 }
 
 type connector struct {
-	driver adbc.Driver
+	db  adbc.Database
+	drv adbc.Driver
 }
 
 // Connect returns a connection to the database. Connect may
@@ -103,17 +105,17 @@ type connector struct {
 //
 // The returned connection is only used by one goroutine at a time.
 func (c *connector) Connect(ctx context.Context) (driver.Conn, error) {
-	cnxn, err := c.driver.Open(ctx)
+	cnxn, err := c.db.Open(ctx)
 	if err != nil {
 		return nil, err
 	}
 
-	return &conn{Conn: cnxn, drv: c.driver}, nil
+	return &conn{Conn: cnxn, drv: c.db}, nil
 }
 
 // Driver returns the underlying Driver of the connector,
 // mainly to maintain compatibility with the Driver method on sql.DB
-func (c *connector) Driver() driver.Driver { return &Driver{c.driver} }
+func (c *connector) Driver() driver.Driver { return Driver{c.drv} }
 
 type Driver struct {
 	Driver adbc.Driver
@@ -129,21 +131,11 @@ type Driver struct {
 //
 // The returned connection is only used by one goroutine at a time.
 func (d Driver) Open(name string) (driver.Conn, error) {
-	opts, err := parseConnectStr(name)
+	connector, err := d.OpenConnector(name)
 	if err != nil {
 		return nil, err
 	}
-	err = d.Driver.SetOptions(opts)
-	if err != nil {
-		return nil, err
-	}
-
-	cnxn, err := d.Driver.Open(context.Background())
-	if err != nil {
-		return nil, err
-	}
-
-	return &conn{Conn: cnxn, drv: d.Driver}, nil
+	return connector.Connect(context.Background())
 }
 
 // OpenConnector expects the same format as driver.Open
@@ -152,12 +144,13 @@ func (d Driver) OpenConnector(name string) (driver.Connector, error) {
 	if err != nil {
 		return nil, err
 	}
-	err = d.Driver.SetOptions(opts)
+
+	db, err := d.Driver.NewDatabase(opts)
 	if err != nil {
 		return nil, err
 	}
 
-	return &connector{d.Driver}, nil
+	return &connector{db, d.Driver}, nil
 }
 
 type ctxOptsKey struct{}
@@ -178,7 +171,7 @@ func GetOptionsFromCtx(ctx context.Context) map[string]string {
 // multiple goroutines. It is assumed to be stateful.
 type conn struct {
 	Conn adbc.Connection
-	drv  adbc.Driver
+	drv  adbc.Database
 }
 
 // Close invalidates and potentially stops any current prepared
@@ -196,6 +189,32 @@ func (c *conn) Close() error {
 	return c.Conn.Close()
 }
 
+func (c *conn) Query(query string, values []driver.Value) (driver.Rows, error) {
+	namedValues := make([]driver.NamedValue, len(values))
+	for i, value := range values {
+		namedValues[i] = driver.NamedValue{
+			// nb: Name field is optional
+			Ordinal: i,
+			Value:   value,
+		}
+	}
+	return c.QueryContext(context.Background(), query, namedValues)
+}
+
+func (c *conn) QueryContext(ctx context.Context, query string, args []driver.NamedValue) (driver.Rows, error) {
+	s, err := c.Conn.NewStatement()
+	if err != nil {
+		return nil, err
+	}
+
+	if err = s.SetSqlQuery(query); err != nil {
+		s.Close()
+		return nil, err
+	}
+
+	return (&stmt{stmt: s}).QueryContext(ctx, args)
+}
+
 // Begin exists to fulfill the Conn interface, but will return an error.
 // Instead, the ConnBeginTx interface is implemented instead.
 //
@@ -617,15 +636,15 @@ func (r *rows) Next(dest []driver.Value) error {
 		case *array.LargeBinary:
 			dest[i] = col.Value(int(r.curRow))
 		case *array.Date32:
-			dest[i] = col.Value(int(r.curRow))
+			dest[i] = col.Value(int(r.curRow)).ToTime()
 		case *array.Date64:
-			dest[i] = col.Value(int(r.curRow))
+			dest[i] = col.Value(int(r.curRow)).ToTime()
 		case *array.Time32:
-			dest[i] = col.Value(int(r.curRow))
+			dest[i] = col.Value(int(r.curRow)).ToTime(col.DataType().(*arrow.Time32Type).Unit)
 		case *array.Time64:
-			dest[i] = col.Value(int(r.curRow))
+			dest[i] = col.Value(int(r.curRow)).ToTime(col.DataType().(*arrow.Time64Type).Unit)
 		case *array.Timestamp:
-			dest[i] = col.Value(int(r.curRow))
+			dest[i] = col.Value(int(r.curRow)).ToTime(col.DataType().(*arrow.TimestampType).Unit)
 		default:
 			return &adbc.Error{
 				Code: adbc.StatusNotImplemented,
@@ -638,6 +657,10 @@ func (r *rows) Next(dest []driver.Value) error {
 	return nil
 }
 
+func (r *rows) ColumnTypeDatabaseTypeName(index int) string {
+	return r.rdr.Schema().Field(index).Type.String()
+}
+
 func (r *rows) ColumnTypeNullable(index int) (nullable, ok bool) {
 	return r.rdr.Schema().Field(index).Nullable, true
 }
@@ -685,16 +708,8 @@ func (r *rows) ColumnTypeScanType(index int) reflect.Type {
 		return reflect.TypeOf([]byte{})
 	case arrow.STRING:
 		return reflect.TypeOf(string(""))
-	case arrow.TIME32:
-		return reflect.TypeOf(arrow.Time32(0))
-	case arrow.TIME64:
-		return reflect.TypeOf(arrow.Time64(0))
-	case arrow.DATE32:
-		return reflect.TypeOf(arrow.Date32(0))
-	case arrow.DATE64:
-		return reflect.TypeOf(arrow.Date64(0))
-	case arrow.TIMESTAMP:
-		return reflect.TypeOf(arrow.Timestamp(0))
+	case arrow.TIME32, arrow.TIME64, arrow.DATE32, arrow.DATE64, arrow.TIMESTAMP:
+		return reflect.TypeOf(time.Time{})
 	}
 	return nil
 }
