@@ -15,7 +15,7 @@
 # specific language governing permissions and limitations
 # under the License.
 
-import importlib.resources
+import functools
 import typing
 
 import adbc_driver_manager
@@ -27,8 +27,38 @@ __all__ = ["connect", "__version__"]
 
 def connect(uri: typing.Optional[str] = None) -> adbc_driver_manager.AdbcDatabase:
     """Create a low level ADBC connection to SQLite."""
-    root = importlib.resources.files(__package__)
-    entrypoint = root.joinpath("libadbc_driver_sqlite.so")
     if uri is None:
-        return adbc_driver_manager.AdbcDatabase(driver=str(entrypoint))
-    return adbc_driver_manager.AdbcDatabase(driver=str(entrypoint), uri=uri)
+        return adbc_driver_manager.AdbcDatabase(driver=_driver_path())
+    return adbc_driver_manager.AdbcDatabase(driver=_driver_path(), uri=uri)
+
+
+@functools.cache
+def _driver_path() -> str:
+    import importlib.resources
+    import pathlib
+    import sys
+
+    driver = "adbc_driver_sqlite"
+
+    # Wheels bundle the shared library
+    root = importlib.resources.files(__package__)
+    # The filename is always the same regardless of platform
+    entrypoint = root.joinpath(f"lib{driver}.so")
+    if entrypoint.is_file():
+        return str(entrypoint)
+
+    # Search sys.prefix + '/lib' (Unix, Conda on Unix)
+    root = pathlib.Path(sys.prefix)
+    for filename in (f"lib{driver}.so", f"lib{driver}.dylib"):
+        entrypoint = root.joinpath("lib", filename)
+        if entrypoint.is_file():
+            return str(entrypoint)
+
+    # Conda on Windows
+    entrypoint = root.joinpath("bin", f"{driver}.dll")
+    if entrypoint.is_file():
+        return str(entrypoint)
+
+    # Let the driver manager fall back to (DY)LD_LIBRARY_PATH/PATH
+    # (It will insert 'lib', 'so', etc. as needed)
+    return driver
